<?php
/**
 * @author      Wployalty (Alagesan)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html
 * @link        https://www.wployalty.net
 * */

namespace Wlr\App\Premium\Helpers;
defined('ABSPATH') or die();

use Wlr\App\Helpers\Base;
use Wlr\App\Helpers\EarnCampaign;
use Wlr\App\Helpers\Woocommerce;

class ProductReview extends Base
{
    public static $instance = null;

    public function __construct($config = array())
    {
        parent::__construct($config);
    }

    function getTotalEarnPoint($point, $rule, $data)
    {
        if (isset($rule->earn_campaign->point_rule) && !empty($rule->earn_campaign->point_rule)) {
            $point_rule = $rule->earn_campaign->point_rule;
            if (self::$woocommerce_helper->isJson($rule->earn_campaign->point_rule)) {
                $point_rule = json_decode($rule->earn_campaign->point_rule);
            }
            if (isset($rule->earn_campaign->id) && !empty($rule->earn_campaign->id)) {
                $data['campaign_id'] = $rule->earn_campaign->id;
            }
            $data['earn_type'] = 'point';
            $status = $this->checkProductReview($data);
            if ($status && isset($point_rule->earn_point)) {
                $point = (int)$point_rule->earn_point;
            }
        }
        return $point;
    }

    function checkProductReview($data)
    {
        if (isset($data['user_email']) && empty($data['user_email'])) {
            if (isset($data['is_message']) && $data['is_message']) {
                return true;
            }
            return false;
        }
        if (isset($data['product_id']) && empty($data['product_id'])) {
            if (isset($data['is_message']) && $data['is_message']) {
                return true;
            }
            return false;
        }
        $userEmail = sanitize_email($data['user_email']);
        $order_ids = $this->get_orders_ids_by_product_id($data['product_id'], $userEmail);
        $status = apply_filters('wlr_check_product_review_order_is_placed', true, $userEmail, $order_ids);
        if (empty($order_ids) && $status) {
            return false;
        }

        $status = false;

        if (isset($data['earn_type']) && in_array($data['earn_type'], array('point', 'coupon'))) {
            $query_data = array(
                'user_email' => array('operator' => '=', 'value' => $userEmail),
                'action_type' => array('operator' => '=', 'value' => $data['action_type']),
                'product_id' => array('operator' => '=', 'value' => (int)$data['product_id']),
                //'campaign_type' => array('operator' => '=', 'value' => $data['earn_type']),
            );
            if (isset($data['campaign_id']) && !empty($data['campaign_id'])) {
                $query_data['campaign_id'] = array('operator' => '=', 'value' => (int)$data['campaign_id']);
            }
            $transaction_data = self::$earn_campaign_transaction_model->getQueryData($query_data, '*', array(), false, true);
            if (empty($transaction_data)) {
                $status = true;
            }
        }
        return $status;
    }

    function get_orders_ids_by_product_id($product_id, $email)
    {
        if ($product_id <= 0 || empty($email)) {
            return array();
        }
        global $wpdb;
        $options = get_option('wlr_settings', '');
        $earning_status = (isset($options['wlr_earning_status']) && !empty($options['wlr_earning_status']) ? $options['wlr_earning_status'] : array('processing', 'completed'));
        if (is_string($earning_status)) {
            $earning_status = explode(',', $earning_status);
        }
        $earning_status = self::$woocommerce_helper->changeToQueryStatus($earning_status);
        $earning_status = "'" . implode("', '", $earning_status) . "'";
        $query = $wpdb->prepare("SELECT DISTINCT woi.order_id FROM {$wpdb->prefix}woocommerce_order_items as woi 
        LEFT JOIN {$wpdb->prefix}woocommerce_order_itemmeta as woim ON woi.order_item_id = woim.order_item_id
        LEFT JOIN {$wpdb->prefix}woocommerce_order_itemmeta as woim1 ON woi.order_item_id = woim1.order_item_id AND woim1.meta_key = %s
        LEFT JOIN {$wpdb->prefix}posts as p ON woi.order_id = p.ID LEFT JOIN {$wpdb->prefix}postmeta as pm ON p.ID = pm.post_id
        WHERE p.post_status IN ( $earning_status ) AND woim.meta_key IN ( %s, %s ) 
          AND woim.meta_value = %s AND pm.meta_key= %s AND pm.meta_value = %s AND woim1.meta_value IS NULL  ORDER BY woi.order_item_id DESC",
            array('_bundled_by', '_product_id', '_variation_id', $product_id, '_billing_email', $email));
        return $wpdb->get_col($query);
    }

    function getTotalEarnReward($reward, $rule, $data)
    {
        if (isset($rule->earn_campaign->point_rule) && !empty($rule->earn_campaign->point_rule)) {
            $point_rule = $rule->earn_campaign->point_rule;
            if (self::$woocommerce_helper->isJson($rule->earn_campaign->point_rule)) {
                $point_rule = json_decode($rule->earn_campaign->point_rule);
            }
            if (isset($rule->earn_campaign->id) && !empty($rule->earn_campaign->id)) {
                $data['campaign_id'] = $rule->earn_campaign->id;
            }
            $data['earn_type'] = 'coupon';
            $status = $this->checkProductReview($data);
            if ($status && isset($point_rule->earn_reward)) {
                $action_reward = $this->getRewardById($point_rule->earn_reward);
                $reward = !empty($action_reward) ? $action_reward : $reward;
            }
        }
        return $reward;
    }

    function applyEarnProductReview($action_data)
    {
        if (!is_array($action_data) || empty($action_data['user_email'])) {
            return false;
        }
        if (empty($action_data['product_id'])) {
            return false;
        }
        $status = false;
        $earn_campaign = EarnCampaign::getInstance();
        $cart_action_list = array('product_review');
        foreach ($cart_action_list as $action_type) {
            $variant_reward = $this->getTotalEarning($action_type, array(), $action_data);
            foreach ($variant_reward as $campaign_id => $v_reward) {
                if (isset($v_reward['point']) && !empty($v_reward['point']) && $v_reward['point'] > 0) {
                    $status = $earn_campaign->addEarnCampaignPoint($action_type, $v_reward['point'], $campaign_id, $action_data);
                }
                if (isset($v_reward['rewards']) && $v_reward['rewards']) {
                    foreach ($v_reward['rewards'] as $single_reward) {
                        $status = $earn_campaign->addEarnCampaignReward($action_type, $single_reward, $campaign_id, $action_data);
                    }
                }
            }
        }
        return $status;
    }

    public static function getInstance(array $config = array())
    {
        if (!self::$instance) {
            self::$instance = new self($config);
        }
        return self::$instance;
    }

    function processMessage($point_rule, $earning)
    {
        $point = isset($earning['point']) && !empty($earning['point']) ? (int)$earning['point'] : 0;
        $rewards = isset($earning['rewards']) && !empty($earning['rewards']) ? (array)$earning['rewards'] : array();
        $available_rewards = '';
        foreach ($rewards as $single_reward) {
            if (is_object($single_reward) && isset($single_reward->display_name)) {
                $available_rewards .= __($single_reward->display_name, 'wp-loyalty-rules') . ',';
            }
        }
        $available_rewards = trim($available_rewards, ',');
        $reward_count = 0;
        if (!empty($available_rewards)) {
            $reward_count = count(explode(',', $available_rewards));
        }
        $display_message = '';
        if (($point > 0 || !empty($available_rewards))) {
            $message = '';
            $review_message = isset($point_rule->review_message) && !empty($point_rule->review_message) ? __($point_rule->review_message, 'wp-loyalty-rules') : '';
            if (!empty($review_message)) {
                $message = '<span class="wlr-product-review-message">' . Woocommerce::getCleanHtml($review_message) . '</span>';
            }
            $point = $this->roundPoints($point);
            $short_code_list = array(
                '{wlr_points}' => $point > 0 ? number_format_i18n($point) : '',
                '{wlr_points_label}' => $this->getPointLabel($point),
                '{wlr_reward_label}' => $this->getRewardLabel($reward_count),
                '{wlr_rewards}' => $available_rewards
            );
            $display_message = $this->processShortCodes($short_code_list, $message);
        }
        return $display_message;
    }

}